package jfd.ale;

import java.io.BufferedReader;
import java.io.FileReader;

/**
 * Utility class designed to make moving between external .ale files
 * and embedded String[] data easy.
 *<P>
 * This class should actually be run by invoking "make_array" from the
 * tools directory.  It reads from standard input and writes to
 * standard output, the only recognized switch is "-compact" which
 * causes the utility to remove all whitespace in the generated array.
 *<P>
 * As an example, suppose you have an ale file "myfile.ale" whose contents
 * you wish to include in your .java code as a String[].  Assume you
 * are working in the examples/simple directory of this distribution.
 *<P>
 * ../../tools/make_array < myfile.ale > temp.txt (unix)
 *<P>
 * ..\..\tools\make_array < myfile.ale > temp.txt (pc)
 *<P>
 * Then (using your favorite text editor) import the temp.txt file into
 * your java code
 */

public class MakeArray
{
   private static final int MAX_CHARS_PER_LINE = 60;

   MakeArray() {}

   private static String[] split(String s, int maxLength)
   {
      int length = s.length();
      int count  = length / maxLength;

      if(length % maxLength != 0)
         count++;

      String array[] = new String[count];

      for(int i = 0; i < count; i ++)
      {
         array[i] = s.substring(
            i * maxLength, Math.min((i + 1) * maxLength, length)
         );
      }

      return(array);
   }

   private static void usage()
   {
      System.out.println("Usage: program {flags} inputFileName");
      System.out.println("flags: -compact, remove whitespace");
      System.exit(0);
   }

   private static String compact(String s)
   {
      StringBuffer sb = new StringBuffer("");

      for(int j = s.length() - 1, i = 0; j >= 0; i ++, j --)
      {
         char c = s.charAt(i);

         switch(c)
         {
            case ' ': case '\t': case '\n': case '\r': case '\f': break;

            default: sb.append(c); break;
         }
      }

      return(sb.toString());
   }

   public static void main(String args[])
   {
      boolean compact = false;
      String fileName = null;

      for(int i = 0; i < args.length; i ++)
      {
         if(args[i].charAt(0) == '-')
         {
            String arg = args[i].substring(1).toLowerCase();

            if(arg.equals("compact"))
               compact = true;
         }
         else
            fileName = args[i];
      }

      if(fileName == null)
         usage();

      BufferedReader reader = null;

      try
      {
         reader = new BufferedReader(new FileReader(fileName));

         System.out.println("   private static final String array[] =");
         System.out.println("   {");

         String s = null;

         while((s = reader.readLine()) != null)
         {
            if(compact)
               s = compact(s);

            int length = s.length();

            if(length <= MAX_CHARS_PER_LINE)
            {
               if(length > 0 || !compact)
                  System.out.println("      \"" + s + "\",");
            }

            else
            {
               String array[] = split(s, MAX_CHARS_PER_LINE);

               for(int i = 0; i < array.length; i ++)
               {
                  if(i > 0)
                     System.out.print("   ");

                  System.out.print("      \"" + array[i] + "\"");

                  System.out.println(i < array.length - 1 ? " +" : ",");
               }
            }
         }

         System.out.println("   };");
      }
      catch(Exception e)
      {
         e.printStackTrace();
      }
      finally
      {
         if(reader != null)
         {
            try { reader.close(); }
            catch(Exception ignored) {}
         }
      }
   }
}
